import os
import sqlite3
import sys

from . import args
from . import db
from . import parser
from . import util

from .ctx import Context


dbconn = None
dbcur  = None
ctx    = None

try:
  parg = args.create()
  try:  # parsing args
    params = parg.parse_args(sys.argv[1:])
    if params.actions is None:
      params.actions = []
  except Exception as e:
    parg.print_help()
    raise Exception("failure while parsing args", e)

  try:  # initializing DB
    dbconn = sqlite3.connect(params.dbpath)
    dbcur  = dbconn.cursor()
    db.try_init(dbcur)
  except Exception as e:
    raise Exception("failure while initializing DB", e)

  ctx = Context(dbcur)
  for action in enumerate(params.actions):
    try:
      action[1](ctx)
    except Exception as e:
      raise Exception(f"failure while executing {action[0]}-th action", e)

except Exception as e:
  print(f"bookeeper: error: {e}", file=sys.stderr)

finally:
  if ctx:
    ctx.finalize()

  if dbcur:
    dbcur.close()

  if dbconn:
    dbconn.commit();
    dbconn.close()
