import argparse

def create():
  p = argparse.ArgumentParser(
    prog="bookeeper",
    description="CLI app for bookkeeping",
    epilog="developed by falsycat <me@falsy.cat>",
  )

  p.add_argument(
    "dbpath",
    help="path to sqlite database file",
    nargs="?",
    default=":memory:",
  )

  p.add_argument(
    "-p", "--permanentize",
    help="apply changes",
    nargs=1,
    action=_PermanentizeAction,
    dest="actions",
  )
  p.add_argument(
    "-r", "--read",
    help="reads all records from the file",
    nargs=1,
    action=_ReadRecordsAction,
    dest="actions",
  )
  p.add_argument(
    "-o", "--output",
    help="changes output file for latter actions",
    nargs=1,
    action=_ChangeOutputAction,
    dest="actions",
  )
  p.add_argument(
    "--sql",
    help="executes SQL command",
    nargs=1,
    action=_SqlAction,
    dest="actions",
  )
  return p

class _PermanentizeAction(argparse.Action):
  def __call__(self, parser, ns, values, option_string=None):
    _push_action(ns, lambda x: x.permanentize())

class _ReadRecordsAction(argparse.Action):
  def __call__(self, parser, ns, values, option_string=None):
    _push_action(ns, lambda x: x.read_records(values[0]))

class _ChangeOutputAction(argparse.Action):
  def __call__(self, parser, ns, values, option_string=None):
    _push_action(ns, lambda x: x.change_output(values[0]))

class _SqlAction(argparse.Action):
  def __call__(self, parser, ns, values, option_string=None):
    _push_action(ns, lambda x: x.sql(values[0]))

def _push_action(ns, act):
  actions = getattr(ns, "actions", None)
  if actions is None:
    actions = []
    setattr(ns, "actions", actions)
  actions.append(act)
