import sqlite3

def try_init(cur):
  cur.execute("""
    CREATE TABLE IF NOT EXISTS entry (
      id   INTEGER    PRIMARY KEY AUTOINCREMENT,
      type VARCHAR(1) NOT NULL CHECK(type IN ("A","D","N","E","R")),
      name TEXT       NOT NULL,
    
      UNIQUE (type, name)
    );
  """)
  cur.execute("""
    CREATE TABLE IF NOT EXISTS tx (
      id      INTEGER PRIMARY KEY AUTOINCREMENT,
      summary TEXT,
      date    DATE NOT NULL,
      time    TIME
    );
  """)
  cur.execute("""
    CREATE TABLE IF NOT EXISTS journal (
      id     INTEGER PRIMARY KEY AUTOINCREMENT,
      tx     INTEGER NOT NULL,
      entry  INTEGER NOT NULL,
      amount INTEGER NOT NULL,
    
      FOREIGN KEY(tx)    REFERENCES tx(id),
      FOREIGN KEY(entry) REFERENCES entry(name)
    );
  """)

def apply(cur, tx):
  cur.execute("""
    INSERT INTO tx(summary, date, time) VALUES (?, ?, ?) RETURNING id
  """, (tx.summary, tx.date, tx.time))
  txid = next(cur)[0]

  for j in tx.journals:
    cur.execute("""
      INSERT INTO entry(type, name) VALUES (?, ?)
      ON CONFLICT DO UPDATE SET id=id
      RETURNING id
    """, j[0:2])
    eid = next(cur)[0]

    cur.execute("""
      INSERT INTO journal(tx, entry, amount) VALUES (?, ?, ?)
    """, (txid, eid, j[2]))
